<?php
////////////////// HTTP and FTP FUNCTIONS //////////////////
/**
 FUNCTION: ffl_HttpGet()
 * Perform a HTTP Get Request.
 *
 * ffl_HttpGet uses fsockopen() to request a given URL via HTTP
 * 1.0 GET and returns a three element array.  On success, array
 * key 'body' contains the body of the request's reply and key
 * 'header' contains the reply's headers.  On error, the keys
 * returned are 'errornumber' and 'errorstring' from
 * fsockopen()'s third and fourth arguments.  In either case,
 * key 'url' contains an array such as returned from parse_url()
 * after the input url has been massaged a bit.
 *
 * {@source }
 *
 * @param string $url URL to fetch.
 * @param boolean $followRedirects Optionally follow 
 * 'location:' in header, default true.
 * @return array 'header', 'body', 'url' OR 'errorstring',
 * 'errornumber', 'url'.
 */
function ffl_HttpGet( $url, $followRedirects=true, $timeout=30 ) {
    $url_parsed = parse_url($url);
    if ( empty($url_parsed['scheme']) ) {
        $url_parsed = parse_url('http://'.$url);
    }
    $rtn['url'] = $url_parsed;

    $port = $url_parsed["port"];
    if ( !$port ) {
        $port = 80;
    }
    $rtn['url']['port'] = $port;
    
    $path = $url_parsed["path"];
    if ( empty($path) ) {
            $path="/";
    }
    if ( !empty($url_parsed["query"]) ) {
        $path .= "?".$url_parsed["query"];
    }
    $rtn['url']['path'] = $path;

    $host = $url_parsed["host"];
    $foundBody = false;

    $out = "GET $path HTTP/1.0\r\n";
    $out .= "Host: $host\r\n";
    $out .= "Connection: Close\r\n\r\n";

    if ( !$fp = @fsockopen($host, $port, $errno, $errstr, $timeout) ) {
        $rtn['errornumber'] = $errno;
        $rtn['errorstring'] = $errstr;
        return $rtn;
    }
    fwrite($fp, $out);
	
	// Defaults
	$body = '';
	$header  = '';
    while (!feof($fp)) {
        $s = fgets($fp, 128);
        if ( $s == "\r\n" ) {
            $foundBody = true;
            continue;
        }
        if ( $foundBody ) {
            $body .= $s;
        } else {
            if ( ($followRedirects) && (stristr($s, "location:") != false) ) {
                $redirect = preg_replace("/location:/i", "", $s);
                return ffl_HttpGet( trim($redirect) );
            }
            $header .= $s;
        }
    }
    fclose($fp);

    $rtn['header'] = trim($header);
    $rtn['body'] = trim($body);
    
    //echo "".$rtn['header']."<BR><BR>";
    //echo "".$rtn['body']."<BR><BR>";
    
    return $rtn;
}

// Similar to ffl_HttpGet, but this time a Post
/* Usage Example: 
  // Build an authcheck XML document 
  $dataAuth  = "<?xml version='1.0' encoding='utf-8'?>".
          "<AuthCheck><UserID>".$campaign_UserID."</UserID><Password>".$campaign_Password."</Password>".
          "<Customer>".$campaign_Customer."</Customer></AuthCheck>";
          
  // Send the command to the a CampaignAuthCheck PHP page for continuing to CampaignDialerServer
  $authresult = ffl_HttpPost(constant("SENDTOIP")
                            .':'
                            .constant("SENDTOPORT")
                            .constant("SENDAUTH"), 
                            $dataAuth );
   $xmldoc = $authresult['body'];

   // Assume an XML document, and process...
   $doc = new DOMDocument();
   $doc->loadXML($xmldoc);

   // Did we authenticate? 
   $Success   = $doc->getElementsByTagName("Success")->item(0)->nodeValue;     
   if ($Success <> "") ...
*/
function ffl_HttpPost( $url, $postdata, $followRedirects=true) 
{
	$url_parsed = parse_url($url);
	if ( empty($url_parsed['scheme']) ) 
	{
		$url_parsed = parse_url('http://'.$url);
	}
	$rtn['url'] = $url_parsed;

	$port = 80;
	if (isset($url_parsed["port"]))
	{
		$port = $url_parsed["port"];
	}
	$rtn['url']['port'] = $port;

	$path='';
	if (isset($url_parsed["path"]))
	{
		$path = $url_parsed["path"];
	}
	if ( empty($path) ) 
	{
		$path="/";
	}		
	
	if ( !empty($url_parsed["query"]) ) 
	{
		$path .= "?".$url_parsed["query"];
	}
	$rtn['url']['path'] = $path;
	$host = $url_parsed["host"];
	$foundBody = false;

	$out =  "POST $path HTTP/1.0\r\n";
	$out .= "Host: $host\r\n";
	$out .= "Content-Length: ".strlen($postdata)."\r\n";
	$out .= "Connection: keep-alive: 3\r\n\r\n";

	// Note: An error could occur here
	if ( !$fp = @fsockopen($host, $port, $errno, $errstr, 30) ) 
	{
		$rtn['body']		= "Error: Read 'errornumber' and 'errorstring' for details";
		$rtn['errornumber'] = $errno;
		$rtn['errorstring'] = $errstr;
		return $rtn;
	}
	fwrite($fp, $out);
	fwrite($fp, $postdata);
	
	$body = '';
	$header = '';
	while (!feof($fp)) 
	{
		$s = fgets($fp, 128);
		if ( $s == "\r\n" ) 
		{
			$foundBody = true;
			continue;
		}
		
		if ( $foundBody ) 
		{
			$body .= $s;
		}
		else 
		{
			if ( ($followRedirects) && (stristr($s, "location:") != false) ) 
			{
				$redirect = preg_replace("/location:/i", "", $s);
				return ffl_HttpGet( trim($redirect) );
			}
			$header .= $s;
		}
	}
	fclose($fp);
	
	// NOTE: This will sit here until the keepalive has expired (eg: 5 seconds)
	
    $rtn['header'] = trim($header);
    $rtn['body'] = trim($body);
    
    //echo "".$rtn['header']."<BR><BR>";
    //echo "".$rtn['body']."<BR><BR>";
    
    return $rtn;
}


//XML post in a Stream, so that the program will timeout, and continue without holding the user forever
// *** RECOMMENDED *** - Go with this approach for XML post and responses
/*
	$result   = ffl_XMLPostStream($mHostName, $data);
	echo $result['body'];
*/
function ffl_XMLPostStream( $url, $postdata) 
{
	$url_parsed = parse_url($url);
	if ( empty($url_parsed['scheme']) ) 
	{
		$url_parsed = parse_url('http://'.$url);
	}
	$rtn['url'] = $url_parsed;

	$port = 80;
	if (isset($url_parsed["port"]))
	{
		$port = $url_parsed["port"];
	}
	$rtn['url']['port'] = $port;
    
	$path='';
	if (isset($url_parsed["path"]))
	{
		$path = $url_parsed["path"];
	}
	if ( empty($path) ) 
	{
		$path="/";
	}		
	
	if ( !empty($url_parsed["query"]) ) 
	{
		$path .= "?".$url_parsed["query"];
	}
	$rtn['url']['path'] = $path;
	$host = $url_parsed["host"];
	$foundBody = false;

	$out =  "POST $path HTTP/1.0\r\n";
	$out .= "Host: $host\r\n";
	$out .= "Content-Length: ".strlen($postdata)."\r\n";
	$out .= "Connection: keep-alive: 3\r\n\r\n";
	
	// Read socket - timeout 10 seconds
	if ( !$fp = @fsockopen($host, $port, $errno, $errstr, 5) ) 
	{
		$rtn['body']		= "Error: Read 'errornumber' and 'errorstring' for details";
		$rtn['errornumber'] = $errno;
		$rtn['errorstring'] = $errstr;
		return $rtn;
	}
	stream_set_timeout($fp, 5);
	fwrite($fp, $out);
	fwrite($fp, $postdata);

	fwrite($fp, $out);
	
	$responseData = '';
	while (!feof($fp))
	{
		$responseData .= fgets($fp, 128);
	}

	// The \r\n\r\n breaks up the body from the head
	$res = split("\r\n\r\n", $responseData);
	$header = $res[0];
	// Possible not to get a body
	$body	= '';
	if (isset($res[1]))
	{
		$body 	= $res[1];
	}
	
	$info = stream_get_meta_data($fp);
	fclose($fp);

	// How did we get here?
	if ($info['timed_out']) 
	{
		// Connection timed out!
		$body = '<?xml version="1.0" encoding="ISO-8859-1"?><RESPONSE><FAILURE result="TIMEOUT">Timeout</FAILURE></RESPONSE>';
	}
	
	$rtn['body'] = trim($body);
	return $rtn;
} // ffl_XMLPostStream


// HTTP Post
// Post, but if no response happens, move on
// -- Use this function to post a command or XML that cannot be waited for.  
// -- A realtime environment may use this when a result is not critical (eg - command to send email)
function ffl_Post( $path, $port, $postdata, $followRedirects=true) 
{
	$rtn['url']['path'] = $path;
	$rtn['url']['port'] = $port;
	$host = $path;
	$foundBody = false;

	$out =  "POST $path HTTP/1.0\r\n";
	$out .= "Host: $host\r\n";
	$out .= "Content-Length: ".strlen($postdata)."\r\n";
	$out .= "Connection: keep-alive: 3\r\n\r\n";
	//echo $out;    
	if ( !$fp = @fsockopen($host, $port, $errno, $errstr, 30) ) 
	{
		$rtn['errornumber'] = $errno;
		$rtn['errorstring'] = $errstr;
		return $rtn;
	}
	fwrite($fp, $out);
	fwrite($fp, $postdata);

	while (!feof($fp)) 
	{
		$s = fgets($fp, 128);
		if ( $s == "\r\n" ) 
		{
			$foundBody = true;
			continue;
		}
		
		if ( $foundBody ) 
		{
			$body .= $s;
		} 
		else 
		{
			if ( ($followRedirects) && (stristr($s, "location:") != false) ) 
			{
				$redirect = preg_replace("/location:/i", "", $s);
				return ffl_HttpGet( trim($redirect) );
			}
			$header .= $s;
		}
	}
	fclose($fp);

	$rtn['header'] = trim($header);
	$rtn['body'] = trim($body);

	//echo "".$rtn['header']."<BR><BR>";
	//echo "".$rtn['body']."<BR><BR>";

	return $rtn;
} // ffl_Post


// FTP put file
function fFTPPutFile($pUrl, $pUserName, $pUserPass,  $pFileRemote, $pFile )
{
	// set up basic connection
	$conn_id = ftp_connect($pUrl);
	// login with username and password
	$login_result = ftp_login($conn_id, $pUserName, $pUserPass);

	// check connection
	if ((!$conn_id) || (!$login_result)) 
	{
		$result = "FAIL: FTP connection has failed!";
		//echo "FTP connection has failed!";
		//echo "Attempted to connect to $pUrl for user $pUserName";
	}
	else 
	{
		//echo "Connected to $pUrl, for user $pUserName";
		// upload the file
		//ftp_delete($conn_id, $pFileRemote);  
		$upload = ftp_put($conn_id, $pFileRemote, $pFile, FTP_BINARY);

		// check upload status
		if (!$upload) 
		{
			$result = "FAIL: FTP upload has failed!";
		}
		else
		{
			//echo "Uploaded $pFile to $pUrl as $pFile";
			$result = "SUCCESS: FTP upload succeeded!";
		}
	}

	// close the FTP stream
	ftp_close($conn_id);

	return $result;
} // fFTPPutFile


// FTP Get file
function fFTPGetFile($pUrl, $pUserName, $pUserPass, $pFile, $pFileRemote )
{
	$result = FALSE; 

	// set up basic connection
	$conn_id = ftp_connect($pUrl);
	// login with username and password
	$login_result = ftp_login($conn_id, $pUserName, $pUserPass);

	// check connection
	if ((!$conn_id) || (!$login_result)) 
	{
		$result = "FAIL: FTP connection has failed!";
		//echo "FTP connection has failed!";
		//echo "Attempted to connect to $pUrl for user $pUserName";
	}
	else 
	{
		//echo "Connected to $pUrl, for user $pUserName";
		// First, see if the file exists 
		$list     = ftp_nlist($conn_id, $pFileRemote); // 
		if ($list[0] != "") 
		{
			$download = ftp_get($conn_id, $pFile, $pFileRemote, FTP_BINARY);

			// check upload status
			if (!$download) 
			{
				$result = FALSE; // "FAIL: FTP download has failed!";
			}
			else
			{
				//echo "Uploaded $pFile to $pUrl as $pFile";
				$result = TRUE; // "SUCCESS: FTP download succeeded!";
			}
		}
		else
		{
			$result = FALSE; //"FAIL: File not found!";
		}
	}

	// close the FTP stream
	ftp_close($conn_id);

	return $result;

} // fFTPPutFile


// Do a HTTP Get from an HTTP resource
// - Similar to above
function fHttpGet( $pUrl, $pPort, $pCmd )
{
	$result = "";
	$fp     = fsockopen($pUrl, (int)$pPort, $errno, $errstr, 30);

	// Catch any errors that may occur...
	// - the one we consistently get is the "Connection Timeout" error.
	if (!$fp)
	{
		$result = "ERROR: ".$errstr." (".$errno.")";
	}
	// If we establish a connection write a simple message.
	else
	{
		$out =  "POST ".$pCmd;
		$out .= " HTTP/1.1\r\n";
		$out .= "Host: ".$pUrl.":".(int)$pPort."\r\n";
		$out .= "Content-Length: 0\r\n";
		$out .= "Keep-Alive: 300\r\n";
		$out .= "Connection: keep-alive:\r\n\r\n";

		// Debug
		echo $out;

		fwrite($fp, $out);

		/////////////////////////////
		// Read the first line, and process
		/////////////////////////////
		$line =  fread($fp, 8192);
		$result .= substr($line,strrpos($line,"\n"))."\n";

		$status_code = array();
		preg_match('/\d\d\d/', $line, $status_code);
		switch( $status_code[0] ) 
		{
		case 200:
			// Success
			// Do a keep alive, and a total of two lines.
			while ($line = fread($fp, 8192))
			{
			   if ($line == false) 
			   {
				  fclose($fp);
			   }
			   else
			   {  
				  // Add the line to the result
				  $result .= $line."\n";
			   }
			}
			break;
			default:
			// Error
			$result = "ERROR: ".$status_code[0];
		}
		fclose($fp);
	}
	return $result;
} // fHttpGet



// Do a HTTP POST to an HTTP resource
function fHttpPost( $pUrl, $pPort, $pCmd, $pData )
{
	$result = "";
	$fp     = fsockopen($pUrl, (int)$pPort, $errno, $errstr, 30);
	// Catch any errors that may occur...
	// - the one we consistently get is the "Connection Timeout" error.
	if (!$fp)
	{
		$result = "ERROR: ".$errstr." (".$errno.")";
	}
	// If we establish a connection write a simple message.
	else
	{
		$out =  "POST ".$pCmd;
		$out .= " HTTP/1.1\r\n";
		$out .= "Host: ".$pUrl.":".(int)$pPort."\r\n";
		$out .= "Content-Length: ".strlen($pData)."\r\n";
		$out .= "Connection: keep-alive: 1\r\n\r\n";
		// Debug
		//echo $out,"<br><br>",$pData;
		// Send Command
		fwrite($fp, $out);
		// Send Body
		fwrite($fp, $pData);
		//echo("Send Completed<br><br>");
		/////////////////////////////
		// Read the first line, and process
		/////////////////////////////
		$line =  fread($fp, 8192);
		//$result .= substr($line,strrpos($line,"\n"))."\n";
		$result = $line;
		//echo "<br>--------------33--------------<br>";
		//echo ($line);
		//echo "<br>--------------33--------------<br>";
		//echo "LINE BBB: ".$line; //$result;
		$status_code = array();
		preg_match('/\d\d\d/', $line, $status_code);
		switch( $status_code[0] ) 
		{
			case 200:
			// Success
			//echo ("Success<br>");
			// Do a keep alive, and a total of two lines.
			while ($line = fread($fp, 8192))
			{
			   if ($line == false) 
			   {
				  fclose($fp);
			   }
			   else
			   {  
				  // Add the line to the result
				  $result .= $line."\n";
				  //echo "LINE2: ".$line."<br>";
			   }
			}
			break;
			default:
			// Error
			$result = "<br>ERROR: ".$status_code[0];
		}
		fclose($fp);
	} 
	//echo ("<br><br>Result: ".$result);
	return $result;
} // fHttpPost

?>